/* File:            $RCSfile: SystemUtils.java,v $
 * Revision:        $Revision: 1.5 $
 * Author:          $Author: svc $
 * Date:            $Date: 2007/05/21 11:56:01 $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package dk.netarkivet.common.utils;

import java.net.InetAddress;
import java.net.UnknownHostException;

/** Miscellanous utilities for getting system resources. */
public class SystemUtils {

    /** Hostname used when no name kan be found. */
    private static final String LOCALHOST = "localhost";

    /**
     * Looks up the IP number of the local host.
     * Note that Java does not guarantee that the result is
     * IPv4 or IPv6.
     * @return the found IP; returns "UNKNOWNIP" if it could not
     * be found.
     */
    public static String getLocalIP() {
        String result;
        try {
            result = InetAddress.getLocalHost().getHostAddress();
        } catch (UnknownHostException e) {
            result = "UNKNOWNIP";
        }
        return result;
    }

    /**
     * Get the first hostname available for this machine, or "localhost" if none
     * are available.
     *
     * @return A hostname, as returned by
     * InetAddress.getLocalHost().getCanonicalHostName()()
     */
    public static String getLocalHostName() {
        String hostname = LOCALHOST;
        try {
            hostname = InetAddress.getLocalHost().getCanonicalHostName();
        } catch (UnknownHostException e) {
            // If no interfaces, use default;
        }
        return hostname;
    }
}
