/* File:       $RCSfile: IngestDomainList.java,v $
 * Revision:   $Revision: 1.4 $
 * Author:     $Author: kfc $
 * Date:       $Date: 2007/02/02 11:51:10 $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package dk.netarkivet.harvester.datamodel;

import javax.servlet.jsp.JspWriter;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.netarkivet.common.exceptions.IOFailure;


public class IngestDomainList {

    protected final Log log = LogFactory.getLog(getClass());
    private DomainDAO dao;

    public IngestDomainList() {
        dao = DomainDAO.getInstance();
    }

    /**
     * Adds all new domains from a newline-separated file of domain names.
     * @param domainList the file containing the domain names
     * @param out a stream to which output can be sent. May be null.
     */
    public void updateDomainInfo(File domainList, JspWriter out) {

        Domain myDomain;
        String domainName;
        BufferedReader in = null;
        int countDomains = 0;
        boolean print = (out != null);
        try {
            in = new BufferedReader(new InputStreamReader(
                    new FileInputStream(domainList), "UTF-8"));

            while ((domainName = in.readLine()) != null) {
                try {
                    if ((countDomains % 10000) == 0) {
                        String msg = "Domain #" + countDomains + ": "
                            + domainName + " added at " + new Date();
                        log.info(msg);
                        if (print) {
                            out.print(msg);
                            out.print("<br/>");
                            out.flush();
                        }
                    }
                    countDomains++;
                    if (Domain.isValidDomainName(domainName)) {
                        if (!dao.exists(domainName)) {
                            myDomain = Domain.getDefaultDomain(domainName);
                            dao.create(myDomain);
                        }
                    } else {
                        log.debug("domain '"
                                    + domainName +
                                    "' is not a valid domain Name");
                        if (print) {
                            out.print("Domænet '" + domainName +
                                      "' er ikke et gyldigt domænenavn");
                            out.print("<br/>");
                            out.flush();
                        }
                    }
                } catch (Exception e) {
                    log.debug("Could not create domain '"
                              + domainName + "'", e);
                    if (print) {
                        out.print("Kunne ikke oprette domænet '"
                                  + domainName + "'" + e.getMessage());
                        out.print("<br/>\n");
                        out.flush();
                    }
                }
            }

        } catch (FileNotFoundException fnf) {
            String msg = "File not found";
            log.debug(msg);
            throw new IOFailure(msg, fnf);
        } catch (IOException io) {
            String msg = " Can't read the domain";
            log.debug(msg);
            throw new IOFailure(msg, io);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                throw new IOFailure("Problem closing input stream", e);
            }
        }
    }

}
