/*$RCSfile: GetRecordTester.java,v $
* $Revision: 1.9 $
* $Date: 2007/02/02 11:51:13 $
* $Author: kfc $
*
* The Netarchive Suite - Software to harvest and preserve websites
* Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package dk.netarkivet.archive.tools;

import dk.netarkivet.archive.bitarchive.distribute.GetMessage;
import dk.netarkivet.common.distribute.Channels;
import dk.netarkivet.common.distribute.JMSConnection;
import dk.netarkivet.common.distribute.NetarkivetMessage;
import dk.netarkivet.common.distribute.arcrepository.BitarchiveRecord;
import dk.netarkivet.testutils.TestMessageListener;
import dk.netarkivet.testutils.preconfigured.MockupJMS;
import dk.netarkivet.testutils.preconfigured.MoveTestFiles;
import dk.netarkivet.testutils.preconfigured.PreserveStdStreams;
import dk.netarkivet.testutils.preconfigured.PreventSystemExit;

import junit.framework.TestCase;
import org.archive.io.arc.ARCConstants;
import org.archive.io.arc.ARCRecord;
import org.archive.io.arc.ARCRecordMetaData;

import javax.jms.Message;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.HashMap;

public class GetRecordTester extends TestCase {
    private static String CONTENT = "This is a test message";
    private PreventSystemExit pse = new PreventSystemExit();
    private PreserveStdStreams pss = new PreserveStdStreams();
    private MoveTestFiles mtf = new MoveTestFiles(TestInfo.DATA_DIR,
            TestInfo.WORKING_DIR);
    private MockupJMS mjms = new MockupJMS();
    TestMessageListener listener;

    public void setUp(){
        mjms.setUp();
        listener = new GetListener(TestInfo.TEST_ENTRY_FILENAME,TestInfo.TEST_ENTRY_OFFSET);
        JMSConnection.getInstance().setListener(Channels.getTheArcrepos(), listener);
        mtf.setUp();
        pss.setUp();
        pse.setUp();
    }
    public void tearDown(){
        pse.tearDown();
        pss.tearDown();
        mtf.tearDown();
        JMSConnection.getInstance().removeListener(Channels.getTheArcrepos(), listener);
        mjms.tearDown();
    }

    public void testMain() {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        System.setOut(new PrintStream(baos));
        GetRecord.main(new String[]{
                TestInfo.INDEX_DIR.getAbsolutePath(),
                TestInfo.TEST_ENTRY_URI});
        System.out.flush();
        String returnedContent = new String(baos.toByteArray());        
        assertEquals("Should return content unchanged, but was: "
                + returnedContent,CONTENT,returnedContent);
    }

    /**
     * This class is a MessageListener that responds to GetMessage,
     * simulating an ArcRepository. It sends a constant response
     * if the GetMessage matches the values given to GetListener's constructor,
     * othrwise it sends a null record as response.
     */
    private static class GetListener extends TestMessageListener {
        private String arcFileName;
        private long offset;
        private ARCRecord myRec;
        public GetListener(String arcFileName, long offset) {
            this.arcFileName = arcFileName;
            this.offset = offset;
            try {
                HashMap<String,Object> map = new HashMap<String,Object>();
                for(Object o : ARCConstants.REQUIRED_VERSION_1_HEADER_FIELDS) {
                    //Some pieces of code check the length field, so make sure this has right value and foo the rest:
                    map.put((String)o,Integer.toString(CONTENT.length()));
                }
                map.put(ARCConstants.ABSOLUTE_OFFSET_KEY, new Long(0L)); //insert dummy offset
                ARCRecordMetaData meta = new ARCRecordMetaData("foo",map);
                InputStream is = new ByteArrayInputStream(CONTENT.getBytes());
                myRec = new ARCRecord(is,meta,0,false,false,false);
            } catch (IOException e) {
                myRec = null;
            }
        }
        public void onMessage(Message o) {
            super.onMessage(o);
            NetarkivetMessage nmsg =
                   received.get(received.size() - 1);
            if (nmsg instanceof GetMessage) {
                GetMessage m = (GetMessage) nmsg;
                if((arcFileName.equals(m.getArcFile())) && (offset == m.getIndex())) {
                    m.setRecord(new BitarchiveRecord(myRec));
                } else {
                    m.setRecord(new BitarchiveRecord(null));
                }
                JMSConnection.getInstance().reply(m);
            }
        }
    };
}
