/*$RCSfile: DBConnectTester.java,v $
* $Revision: 1.11 $
* $Date: 2007/03/08 09:15:47 $
* $Author: lc $
*
* The Netarchive Suite - Software to harvest and preserve websites
* Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package dk.netarkivet.harvester.datamodel;

import java.io.FileInputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.logging.LogManager;

/**
 * Test class for the Database utilities in DBConnect,
 * especially the ones related to backup of the database.
 *
 */


public class DBConnectTester extends DataModelTestCase {
    public DBConnectTester(String s) {
        super(s);
    }

    public void setUp() throws Exception {
        super.setUp();
        FileInputStream fis = new FileInputStream("tests/dk/netarkivet/testlog.prop");
        LogManager.getLogManager().reset();
        LogManager.getLogManager().readConfiguration(fis);
        fis.close();
    }

    public void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * Simple test if DBConnect.getDBConnection() works or not.
     * Uses Settings.DB_URL set in DataModelTestCase.SetUp()
     */
     public void testGetDBConnection() {
         DBConnect.getDBConnection();
     }

    /** Check that the connection has the expected setup.
     * @throws SQLException
     */
    public void testAutocommitOn() throws SQLException {
        Connection c = DBConnect.getDBConnection();
        assertEquals("Connection should have transaction level READ_COMMITTED",
                     Connection.TRANSACTION_READ_COMMITTED, c.getTransactionIsolation());
        assertTrue("Connection should have autocommit on",
                   c.getAutoCommit());
    }

    /** Check that read locks are released after use.
     * @throws InterruptedException
     */
    public void testLockRelease() throws InterruptedException {
        // Go-ahead ticker
        final int[] state = new int[1];
        final Throwable[] error1 = new Throwable[1];
        Thread t1 = new Thread() {
            public void run() {
                try {
                    waitForState(0, state);
                    Connection c = DBConnect.getDBConnection();
                    c.setAutoCommit(false);
                    c.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
                    DBConnect.selectLongList("SELECT domain_id FROM domains");
                    state[0] = 1;
                    waitForState(2, state);
                } catch (Throwable e) {
                    error1[0] = e;
                    state[0] = -1;
                    return;
                }

            }
        };
        final Throwable[] error2 = new Throwable[1];
        Thread t2 = new Thread() {
            public void run() {
                try {
                    waitForState(1, state);
                    Connection c = DBConnect.getDBConnection();
                    c.setAutoCommit(false);
                    PreparedStatement s = c.prepareStatement("DELETE FROM domains WHERE name = ?");
                    s.setString(1, "netarkivet.dk");
                    s.executeUpdate();
                    c.commit();
                    state[0] = 2;
                } catch (Throwable e) {
                    error2[0] = e;
                    state[0] = -1;
                    return;
                }
            }
        };
        t1.start();
        t2.start();
        while (t1.isAlive() && t2.isAlive()) {
            Thread.sleep(10);
        }
        if (error1[0] != null) {
            System.out.println(error1[0]);
            error1[0].printStackTrace();
        }
        if (error2[0] != null) {
            System.out.println(error2[0]);
            error2[0].printStackTrace();
        }
        assertEquals("Should have no exception from thread 1",
                     null, error1[0]);
        assertNull("Should have no exception from thread 2",
                   error2[0]);
    }

    static void waitForState(int state, int[] stateref) {
        while (stateref[0] != state && stateref[0] != -1) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // Ignored, just retesting anyway.
            }
        }
        if (stateref[0] == -1) {
            throw new Error("Other thread failed");
        }
    }
}